using System;
using System.IO;

namespace SBU.HTRRL.FlowPattern.DataAnalysis
{
	/// <summary>
	/// Summary description for Cls_Uncertainty.
	/// </summary>
	public class SourceData
	{
		private FileStream fileIn;
		private StreamReader fileStringIn;
		private static int groupNum=20;
		private static int thermalcoupleNum=15;
		private string[] originalData=new string[800]; //from the data collecting programm (VB 6.0), number=8+20*38=768
		private bool[] validDataFlag=new bool[groupNum];
		private int validDataNum;
		private double thermalLoss1; // average of thermalLossArray1
		private double thermalLoss2; // average of thermalLossArray2 and get rid of the effect of a Mix. and a Min. heating section wall temperature due to the failure of the thermocouple wires
		private double[] thermalLossArray1=new double[groupNum]; //thermal loss between the preheater and the test section from the data collecting programm (VB 6.0)
		private double[] thermalLossArray2=new double[groupNum]; //thermal loss between the inlet of the test section and the observation point from the data collecting programm (VB 6.0)
		private double[,] tt=new double[thermalcoupleNum,groupNum]; //temperatures at the tube wall of the heating section 
		private string diameter;
		private double d,l; //l is the length of the heating section
		private string patternFileName;
		private string flowPattern;
		private double[] m=new double[groupNum];
		private double[] power1=new double[groupNum]; //heating power of the preheater
		private double[] power2=new double[groupNum]; //heating power of the heating section
		private double[] dp=new double[groupNum]; //pressure drop at the heating section
		private double[] p2=new double[groupNum];
		private double[] t2=new double[groupNum];
		private double[] p3=new double[groupNum];
		private double[] t3=new double[groupNum];
		private double[] p4=new double[groupNum];
		private double[] t4=new double[groupNum];
		private double[] p0=new double[groupNum];
		private double[] t0=new double[groupNum];
		private double[] p=new double[groupNum]; // pressure at the observation point
		private double[] t=new double[groupNum]; // temperature at the observation point

		protected SourceData(string fileName)
		{
			try
			{
				fileIn = new FileStream(fileName,FileMode.Open);
				fileStringIn=new StreamReader(fileIn);
				int i=0;
				do
				{
					originalData[i]=fileStringIn.ReadLine();
					i++;
				}while (originalData[i-1]!=null);
				fileStringIn.Close();
			}
			catch(FileNotFoundException exc)
			{
				Console.WriteLine(exc.Message);
				return;
			}
			catch
			{
				Console.WriteLine("Cannot open file.");
				return;
			}

			patternFileName=originalData[0].Remove(0,28);
			patternFileName=patternFileName.Trim();
			flowPattern=originalData[1].Remove(0,28);
			flowPattern=flowPattern.Trim();
			diameter=originalData[4].Remove(0,28);
			d=double.Parse(diameter)/1000; //(m)
			l=double.Parse(originalData[5].Remove(0,28))/1000; // (m)
			thermalLoss1=0; // (W)		
			for (int i=0; i<groupNum;i++)
			{
				thermalLossArray1[i]=double.Parse(originalData[16+38*i].Remove(0,28)); // (W)
				thermalLossArray2[i]=double.Parse(originalData[24+38*i].Remove(0,28)); // (W)
				p2[i]=double.Parse(originalData[12+38*i].Remove(0,28))*100000; //(Pa-a)
				t2[i]=double.Parse(originalData[13+38*i].Remove(0,28))+273.15; //(K)
				m[i]=double.Parse(originalData[14+38*i].Remove(0,28))/3600; //(kg/s)
				power1[i]=double.Parse(originalData[15+38*i].Remove(0,28)); //(W)
				p3[i]=double.Parse(originalData[18+38*i].Remove(0,28))*100000; //(Pa-a)
				t3[i]=double.Parse(originalData[19+38*i].Remove(0,28))+273.15; //(K)
				dp[i]=double.Parse(originalData[20+38*i].Remove(0,28))*100000; //(Pa)
				p4[i]=double.Parse(originalData[21+38*i].Remove(0,28))*100000; //(Pa-a)
				t4[i]=double.Parse(originalData[22+38*i].Remove(0,28))+273.15; //(K)
				power2[i]=double.Parse(originalData[23+38*i].Remove(0,28)); //(W)
				p0[i]=double.Parse(originalData[41+38*i].Remove(0,28))*100000; //(Pa-a)
				t0[i]=double.Parse(originalData[42+38*i].Remove(0,28))+273.15; //(K)
				p[i]=double.Parse(originalData[44+38*i].Remove(0,28))*100000; //(Pa-a)
				t[i]=double.Parse(originalData[45+38*i].Remove(0,28))+273.15; //(K)
				thermalLoss1=thermalLoss1+thermalLossArray1[i]/groupNum; // (W)
				for (int j=0; j<thermalcoupleNum; j++)
				{
					tt[j,i]=double.Parse(originalData[25+j+38*i].Remove(0,28));  //(C)
				}
				validDataFlag[i]=true;
			}
			this.checkThermalLoss2();
			this.checkDataGroup();
		}

		private void checkThermalLoss2()
		{
			double ttMin,ttMax;
			double ttTotal,ttAverage;
			double thermalLoss2LengthCoefficient=0;
			double thermalLoss2Coefficient=0;
			double [] calibratedThermalLossArray2=new double[groupNum];
			switch (diameter)
			{
				case "1.10":
					thermalLoss2LengthCoefficient=2.125;
					thermalLoss2Coefficient=0.048;
					break;
				case "2.01":
					thermalLoss2LengthCoefficient=2.15;
					thermalLoss2Coefficient=0.04;
					break;
				case "2.88":
					thermalLoss2LengthCoefficient=2.35;
					thermalLoss2Coefficient=0.044;
					break;
				case "4.26":
					thermalLoss2LengthCoefficient=1.65;
					thermalLoss2Coefficient=0.068;
					break;
			}
			thermalLoss2=0; // (W)		
			for (int i=0; i<groupNum;i++)
			{
				ttMin=tt[0,i];
				ttMax=tt[0,i];
				ttTotal=tt[0,i];
				for (int j=1; j<thermalcoupleNum; j++)
				{
					ttMin=ttMin>tt[j,i]?tt[j,i]:ttMin;  //(C)
					ttMax=ttMax<tt[j,i]?tt[j,i]:ttMax;  //(C)
					ttTotal=ttTotal+tt[j,i];
				}
				ttAverage=(ttTotal-ttMin-ttMax)/(thermalcoupleNum-2);
				calibratedThermalLossArray2[i]=thermalLossArray2[i]+thermalLoss2LengthCoefficient*thermalLoss2Coefficient*(ttMin+ttMax-2*ttAverage)/thermalcoupleNum;		
				thermalLoss2=thermalLoss2+calibratedThermalLossArray2[i]/groupNum; // (W)
			}
		}

		private void checkData(double[] checkedData)
		{
			double totalValue;
			double averageValue;
			double totalDeviation;
			double standardDeviation;
			double[] rejectingCoefficient={
											  0,0,0,
											  1.38,1.54,1.65,1.73,1.80,1.87,1.91,
											  1.96,1.96+1*(2.13-1.96)/5,1.96+2*(2.13-1.96)/5,1.96+3*(2.13-1.96)/5,1.96+4*(2.13-1.96)/5,
											  2.13,2.13+1*(2.24-2.13)/5,2.13+2*(2.24-2.13)/5,2.13+3*(2.24-2.13)/5,2.13+4*(2.24-2.13)/5,
											  2.24,2.24+1*(2.33-2.24)/5,2.24+2*(2.33-2.24)/5,2.24+3*(2.33-2.24)/5,2.24+4*(2.33-2.24)/5,
											  2.33,2.33+1*(2.57-2.33)/25,2.33+2*(2.57-2.33)/25,2.33+3*(2.57-2.33)/25,2.33+4*(2.57-2.33)/25,2.33+5*(2.57-2.33)/25
										  };
			double[] deviationCoefficient=new double[groupNum];

			validDataNum=0;
			totalValue=0;
			totalDeviation=0;
			for (int i=0; i<checkedData.Length; i++)
			{
				if (validDataFlag[i]==true)
				{
					totalValue+=checkedData[i];
					validDataNum+=1;
				}
			}
			averageValue=totalValue/validDataNum;

			for (int i=0; i<checkedData.Length; i++)
			{
				if (validDataFlag[i]==true)
				{
					totalDeviation+=Math.Pow((checkedData[i]-averageValue),2);
				}
			}
			standardDeviation=Math.Sqrt(totalDeviation/(validDataNum-1));

			for (int i=0; i<checkedData.Length; i++)
			{
				if (validDataFlag[i]==true)
				{	
					deviationCoefficient[i]=Math.Abs((checkedData[i]-averageValue)/standardDeviation);
					if (deviationCoefficient[i]>=rejectingCoefficient[validDataNum])
					{
						validDataFlag[i]=false;
						validDataNum-=1;
					}
				}
			}
			return;
		}

		private void checkDataGroup()
		{
			this.checkData(p2);
			this.checkData(t2);
			this.checkData(m);
			this.checkData(power1);
			this.checkData(p3);
			this.checkData(t3);
			this.checkData(dp);
			this.checkData(p4);
			this.checkData(t4);
			this.checkData(power2);
			this.checkData(p0);
			this.checkData(t0);
			this.checkData(p);
			this.checkData(t);
			return;
		}

		public string PatternFileName
		{
			get
			{
				return patternFileName;
			}
		}
		public string FlowPattern
		{
			get
			{
				return flowPattern;
			}
		}

		public double D //(m)
		{
			get
			{
				return d;
			}
		}
		
		public double L // (m)
		{
			get
			{
				return l;
			}
		}
		
		public double ThermalLoss1 // (W)
		{
			get
			{
				return thermalLoss1;
			}
		}
		
		public double ThermalLoss2 // (W)
		{
			get
			{
				return thermalLoss2;
			}
		}
		
		public double M //(kg/s)
		{
			get
			{
				double mTotal=0;
				double flowRate;
				for (int i=0; i<groupNum;i++)
				{
					mTotal+=(validDataFlag[i]==true?m[i]:0);
				}	
				flowRate=mTotal/validDataNum;
				return flowRate;
			}
		}
		public double Power1 // (W)
		{
			get
			{
				double power1Total=0;
				double preheaterPower;
				for (int i=0; i<groupNum;i++)
				{
					power1Total+=(validDataFlag[i]==true?power1[i]:0);
				}	
				preheaterPower=power1Total/validDataNum;
				return preheaterPower;
			}
		}
		public double Power2 // (W)
		{
			get
			{
				double power2Total=0;
				double testsectionPower;
				for (int i=0; i<groupNum;i++)
				{
					power2Total+=(validDataFlag[i]==true?power2[i]:0);
				}	
				testsectionPower=power2Total/validDataNum;
				return testsectionPower;
			}
		}
		public double DP //(Pa)
		{
			get
			{
				double dpTotal=0;
				double differentPressure;
				for (int i=0; i<groupNum;i++)
				{
					dpTotal+=(validDataFlag[i]==true?dp[i]:0);
				}	
				differentPressure=dpTotal/validDataNum;
				return differentPressure;
			}
		}
		public double P2 //(Pa-a)
		{
			get
			{
				double p2Total=0;
				double preheaterInletPressure;
				for (int i=0; i<groupNum;i++)
				{
					p2Total+=(validDataFlag[i]==true?p2[i]:0);
				}	
				preheaterInletPressure=p2Total/validDataNum;
				return preheaterInletPressure;
			}
		}
		public double T2 //(K)
		{
			get
			{
				double t2Total=0;
				double preheaterInletTemperature;
				for (int i=0; i<groupNum;i++)
				{
					t2Total+=(validDataFlag[i]==true?t2[i]:0);
				}	
				preheaterInletTemperature=t2Total/validDataNum;
				return preheaterInletTemperature;
			}
		}
		public double P3 //(Pa-a)
		{
			get
			{
				double p3Total=0;
				double heatingSectionInletPressure;
				for (int i=0; i<groupNum;i++)
				{
					p3Total+=(validDataFlag[i]==true?p3[i]:0);
				}	
				heatingSectionInletPressure=p3Total/validDataNum;
				return heatingSectionInletPressure;
			}
		}
		public double T3 //(K)
		{
			get
			{
				double t3Total=0;
				double heatingSectionInletTemperature;
				for (int i=0; i<groupNum;i++)
				{
					t3Total+=(validDataFlag[i]==true?t3[i]:0);
				}	
				heatingSectionInletTemperature=t3Total/validDataNum;
				return heatingSectionInletTemperature;
			}
		}

		public double P4 //(Pa-a)
		{
			get
			{
				double p4Total=0;
				double heatingSectionOutletPressure;
				for (int i=0; i<groupNum;i++)
				{
					p4Total+=(validDataFlag[i]==true?p4[i]:0);
				}	
				heatingSectionOutletPressure=p4Total/validDataNum;
				return heatingSectionOutletPressure;
			}
		}
		public double T4 //(K)
		{
			get
			{
				double t4Total=0;
				double heatingSectionOutletTemperature;
				for (int i=0; i<groupNum;i++)
				{
					t4Total+=(validDataFlag[i]==true?t4[i]:0);
				}	
				heatingSectionOutletTemperature=t4Total/validDataNum;
				return  heatingSectionOutletTemperature;
			}
		}

		public double P0 //(Pa-a)
		{
			get
			{
				double p0Total=0;
				double observationSectionOutletPressure;
				for (int i=0; i<groupNum;i++)
				{
					p0Total+=(validDataFlag[i]==true?p0[i]:0);
				}	
				observationSectionOutletPressure=p0Total/validDataNum;
				return observationSectionOutletPressure;
			}
		}
		public double T0 //(K)
		{
			get
			{
				double t0Total=0;
				double observationSectionOutletTemperature;
				for (int i=0; i<groupNum;i++)
				{
					t0Total+=(validDataFlag[i]==true?t0[i]:0);
				}	
				observationSectionOutletTemperature=t0Total/validDataNum;
				return  observationSectionOutletTemperature;
			}
		}

		public double P //(Pa-a)
		{
			get
			{
				double pTotal=0;
				double observationPointpressure;
				for (int i=0; i<groupNum;i++)
				{
					pTotal+=(validDataFlag[i]==true?p[i]:0);
				}	
				observationPointpressure=pTotal/validDataNum;
				return observationPointpressure;
			}
		}
		public double T //(K)
		{
			get
			{
				double tTotal=0;
				double observationPointtemperature;
				for (int i=0; i<groupNum;i++)
				{
					tTotal+=(validDataFlag[i]==true?t[i]:0);
				}	
				observationPointtemperature=tTotal/validDataNum;
				return  observationPointtemperature;
			}
		}
	}

	public class TargetData:SourceData
	{
		//public SourceData Data;
		private R134aProperty Point2;
		private R134aProperty Point3;
		private R134aProperty Point3Sat;
		private R134aProperty Point4;
		private R134aProperty Point4Sat;
		private R134aProperty Point0;
		private R134aProperty Point0Sat;
		private R134aProperty PointObservation;
		private R134aProperty PointObservationSat;

		public TargetData(string fileName):base(fileName)
		{
			double t3Sat,t4Sat,t0Sat,tSat;
			Point2=new R134aProperty(this.T2,this.P2);
			Point3=new R134aProperty(this.T3,this.P3);
			t3Sat=Point3.SaturatedTemperature;
			Point3Sat=new R134aProperty(t3Sat,this.P3);
			Point4=new R134aProperty(this.T4,this.P4);
			t4Sat=Point4.SaturatedTemperature;
			Point4Sat=new R134aProperty(t4Sat,this.P4);
			Point0=new R134aProperty(this.T0,this.P0);
			t0Sat=Point0.SaturatedTemperature;
			Point0Sat=new R134aProperty(t0Sat,this.P0);
			PointObservation=new R134aProperty(this.T,this.P);
			tSat=PointObservation.SaturatedTemperature;
			PointObservationSat=new R134aProperty(tSat,this.P);
		}

		public string getInletFluidState() // point 3
		{
			string state;
			state=this.Point3.State;
			return state;
		}
		public double getInletQuality() // point 3
		{
			double inletEnthalpy,inletQuality;
			if (Point3.State!="liquid")
				inletEnthalpy = (this.M*Point2.LiquidEnthalpy+this.Power1+this.ThermalLoss1) / this.M;
			else
				inletEnthalpy = Point3.LiquidEnthalpy;
            if (inletEnthalpy<=Point3Sat.LiquidEnthalpy)
				inletQuality=0;
			else if (inletEnthalpy>=Point3Sat.VapourEnthalpy)
				inletQuality=1;
			else
				inletQuality = (inletEnthalpy - Point3Sat.LiquidEnthalpy) / (Point3Sat.VapourEnthalpy - Point3Sat.LiquidEnthalpy);
			return inletQuality;
		}

		public double getOutletQuality() // observation point
		{
			double outletEnthalpy,outletQuality;
			if (Point3.State!="liquid")
				outletEnthalpy = (this.M*Point2.LiquidEnthalpy+this.Power1+this.Power2+this.ThermalLoss1+this.ThermalLoss2) / this.M;
			else
				outletEnthalpy = (this.M*Point3.LiquidEnthalpy+this.Power2+this.ThermalLoss2) / this.M;
			if (outletEnthalpy<=PointObservationSat.LiquidEnthalpy)
				outletQuality=0;
			else if (outletEnthalpy>=PointObservationSat.VapourEnthalpy)
				outletQuality=1;
			else
				outletQuality = (outletEnthalpy - PointObservationSat.LiquidEnthalpy) / (PointObservationSat.VapourEnthalpy - PointObservationSat.LiquidEnthalpy);
			return outletQuality;
		}

		public double getInletLiquidVelocity() // point 3
		{
			double inletLiquidVelocity;
			inletLiquidVelocity = 4 * (1 - this.getInletQuality()) * this.M / (Point3.LiquidDensity * Math.PI * Math.Pow(this.D,2));
			return inletLiquidVelocity;
		}

		public double getInletVapourVelocity() // point 3
		{
			double inletVapourVelocity;
			inletVapourVelocity = 4 * this.getInletQuality() * this.M / (Point3.VapourDensity * Math.PI * Math.Pow(this.D,2));
			return inletVapourVelocity;
		}

		public double getOutletLiquidVelocity() // observation point
		{
			double outletLiquidVelocity;
			outletLiquidVelocity = 4 * (1 - this.getOutletQuality()) * this.M / (PointObservation.LiquidDensity * Math.PI * Math.Pow(this.D,2));
			return outletLiquidVelocity;
		}

		public double getOutletVapourVelocity() // observation point
		{
			double outletVapourVelocity;
			outletVapourVelocity = 4 * this.getOutletQuality() * this.M / (PointObservation.VapourDensity * Math.PI * Math.Pow(this.D,2));
			return outletVapourVelocity;
		}

		public double getMassFlux()
		{
			double massFlux;
			massFlux = 4*this.M/ (Math.PI*this.D*this.D);
			return massFlux;
		}

		public double getHeatFlux() //heating section
		{
			double heatFlux;
			heatFlux = this.Power2/ (Math.PI*this.D*this.L);
			return heatFlux;
		}
		
	}
}
