using System;
using System.Drawing;

namespace SBU.HTRRL.FlowPattern.DataAnalysis
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class PatternBoundary
	{
		private string Fluid;
		private double p,d;
		private R134aProperty saturatedProperty;
		private AirWater1bar25CProperty referenceProperty;
		private const double g=9.8065;
		private const double LDratio110=200; // Taitel(1980), L/D Ratio, where ID=1.10mm, L is the distance traversed by the mixture to the observing point.
		private const double LDratio201=150; // Taitel(1980), L/D Ratio, where ID=2.01mm, L is the distance traversed by the mixture to the observing point.
		private const double LDratio288=125; // Taitel(1980), L/D Ratio, where ID=2.88mm, L is the distance traversed by the mixture to the observing point.
		private const double LDratio426=105; // Taitel(1980), L/D Ratio, where ID=4.26mm, L is the distance traversed by the mixture to the observing point.
		public static double LDratioDefault=100; // Taitel(1980), L/D Ratio, where ID=4.26mm, L is the distance traversed by the mixture to the observing point.
		private const double criticalVoidFraction0=0.25; // Taitel(1980), bubbly - slug
		private const double criticalVoidFraction1=0.52; // Taitel(1980), dispersed bubble - churn
		private const double criticalVoidFraction2=0.3; // Mishima & Ishii(1984), bubble - slug
		private const double criticalVoidFraction3=0.74;  //McQuillan & Whalley(1985), dispersed bubble - churn
		public static double plugChurnTransitionConstant=1.0;  //McQuillan & Whalley(1985), plug - churn
		private const double criticalDimensionlessUgs=1.0;  //McQuillan & Whalley(1985), from annular
		public const int GridNumber=1001;
		public static double MinUgs=0.01; //FrmSetCoordinate.Xmin;
		public static double MaxUgs=100; //FrmSetCoordinate.Xmax;
		private double Xstep=(Math.Log10(MaxUgs)-Math.Log10(MinUgs))/(GridNumber-1);
		public static double MinUls=0.01; //FrmSetCoordinate.Ymin;
		public static double MaxUls=10; //FrmSetCoordinate.Ymax;
		private double Ystep=(Math.Log10(MaxUls)-Math.Log10(MinUls))/(GridNumber-1);
		private double Rol,Rog; //density
		private double Mul,Mug,Mulref; //viscosity
		private double St,Stref; //surface tension
		public readonly PointF[] BubblyDispersedbubbleBoundary=new PointF[GridNumber]; //Unified Model, Taitel 1990
		public readonly PointF[] DispersedbubbleChurnBoundary=new PointF[GridNumber]; //Unified Model, Taitel 1990
		public readonly PointF[] IntermitentChurnBoundary=new PointF[GridNumber]; //Unified Model, Taitel 1990
		public readonly PointF[] ChurnAnnularBoundary1=new PointF[GridNumber]; //Unified Model, Taitel 1990
		public readonly PointF[] ChurnAnnularBoundary2=new PointF[GridNumber]; //Unified Model, Taitel 1990
		public readonly PointF[] ChurnAnnularBoundary=new PointF[GridNumber]; //Unified Model, Taitel 1990
		public readonly PointF[] BubblySlugBoundary=new PointF[GridNumber]; //Unified Model, Taitel 1990
		public readonly PointF[] ChurnAnnularBoundaryB=new PointF[GridNumber]; //Taitel et al. Model, 1980
		public readonly PointF[] DispersedbubbleBubblyBoundaryB=new PointF[GridNumber]; //Taitel et al. Model, 1980
		public readonly PointF[] DispersedbubbleChurnBoundaryB=new PointF[GridNumber]; //Taitel et al. Model, 1980
		public readonly PointF[] BubblySlugBoundaryB=new PointF[GridNumber]; //Taitel et al. Model, 1980
		public readonly PointF[] SlugChurnBoundaryB=new PointF[GridNumber]; //Taitel et al. Model, 1980
		public readonly PointF[] BubblySlugBoundaryC=new PointF[GridNumber]; //Mishima & Ishii Model, 1984
		public readonly PointF[] SlugChurnBoundaryC=new PointF[GridNumber]; //Mishima & Ishii Model, 1984
		public readonly PointF[] SlugAnnularBoundaryC=new PointF[GridNumber]; //Mishima & Ishii Model, 1984
		public readonly PointF[] ChurnAnnularBoundaryC=new PointF[GridNumber]; //Mishima & Ishii Model, 1984
		public readonly PointF[] BubblyPlugChurnBoundaryD=new PointF[GridNumber]; //McQuillan & Whalley Model, 1985
		public readonly PointF[] DispersedbubbleChurnBoundaryD=new PointF[GridNumber]; //McQuillan & Whalley Model, 1985
		public readonly PointF[] FromDispersedbubbleBoundaryD=new PointF[GridNumber]; //McQuillan & Whalley Model, 1985
		public readonly PointF[] PlugChurnBoundaryD=new PointF[GridNumber]; //McQuillan & Whalley Model, 1985
		public readonly PointF[] FromAnnularBoundaryD=new PointF[GridNumber]; //McQuillan & Whalley Model, 1985
		public readonly PointF[] BubblySlugBoundaryE=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] DispersedbubbleBubblyBoundaryE=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] DispersedbubbleChurnBoundaryE=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] SlugChurnBoundaryE1=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] SlugChurnBoundaryE2=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] SlugChurnBoundaryE3=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] ChurnAnnularBoundaryE1=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] ChurnAnnularBoundaryE2=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] BubblySlugBoundaryF=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] DispersedbubbleBubblyBoundaryF=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] DispersedbubbleChurnBoundaryF=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] SlugChurnBoundaryF1=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] SlugChurnBoundaryF2=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] ChurnAnnularBoundaryF=new PointF[GridNumber]; //Chen Model, 2006
		public readonly PointF[] ConstantQualityBoundary=new PointF[GridNumber];
		public readonly PointF[][] ConstantQualitiesBoundary=new PointF[FrmFlowPatternGraph.Quality.Length][];
		public readonly PointF[][] ConstantMassFluxBoundary=new PointF[FrmFlowPatternGraph.MassFlux.Length][];
		private int maxIndexOfBubblyDispersedbubbleBoundary;
		private double maxUlsOfBubblyDispersedbubbleBoundary;
		private double dispersedbubbleChurnBoundaryStartPoint;

		public PatternBoundary(double P,double D) //P(Pa-a),D(m)
		{
			Fluid="R134a";
			p=P;
			d=D;
			referenceProperty =new AirWater1bar25CProperty();
			R134aProperty R134aProperty=new R134aProperty(273.15,P);
			saturatedProperty=new R134aProperty(R134aProperty.SaturatedTemperature,P);
			Rol=saturatedProperty.LiquidDensity;
			Rog=saturatedProperty.VapourDensity;
			Mul=saturatedProperty.SaturatedLiquidViscosity;
			Mug=saturatedProperty.SaturatedVapourViscosity;
			St=saturatedProperty.SurfaceTension;
			Mulref=referenceProperty.LiquidViscosity;
			Stref=referenceProperty.SurfaceTension;
		}

		public PatternBoundary(double D, string Condition) //D(m)
		{
			d=D;
			referenceProperty =new AirWater1bar25CProperty();
			Mulref=referenceProperty.LiquidViscosity;
			Stref=referenceProperty.SurfaceTension;
			switch (Condition)
			{
				case "AirWater1bar0C":
				{
					AirWater1bar0CProperty FluidProperty=new AirWater1bar0CProperty();
					Rol=FluidProperty.LiquidDensity;
					Rog=FluidProperty.VapourDensity;
					Mul=FluidProperty.LiquidViscosity;
					Mug=FluidProperty.VapourViscosity;
					St=FluidProperty.SurfaceTension;
					Fluid="Air-Water";
					break;
				}
				case "AirWater1bar20C":
				{
					AirWater1bar20CProperty FluidProperty=new AirWater1bar20CProperty();
					Rol=FluidProperty.LiquidDensity;
					Rog=FluidProperty.VapourDensity;
					Mul=FluidProperty.LiquidViscosity;
					Mug=FluidProperty.VapourViscosity;
					St=FluidProperty.SurfaceTension;
					Fluid="Air-Water";
					break;
				}
				case "AirWater1bar25C":
				{
					AirWater1bar25CProperty FluidProperty=new AirWater1bar25CProperty();
					Rol=FluidProperty.LiquidDensity;
					Rog=FluidProperty.VapourDensity;
					Mul=FluidProperty.LiquidViscosity;
					Mug=FluidProperty.VapourViscosity;
					St=FluidProperty.SurfaceTension;
					Fluid="Air-Water";
					break;
				}
				case "AirWater1bar50C":
				{
					AirWater1bar50CProperty FluidProperty=new AirWater1bar50CProperty();
					Rol=FluidProperty.LiquidDensity;
					Rog=FluidProperty.VapourDensity;
					Mul=FluidProperty.LiquidViscosity;
					Mug=FluidProperty.VapourViscosity;
					St=FluidProperty.SurfaceTension;
					Fluid="Air-Water";
					break;
				}
				case "AirWater1bar100C":
				{
					AirWater1bar100CProperty FluidProperty=new AirWater1bar100CProperty();
					Rol=FluidProperty.LiquidDensity;
					Rog=FluidProperty.VapourDensity;
					Mul=FluidProperty.LiquidViscosity;
					Mug=FluidProperty.VapourViscosity;
					St=FluidProperty.SurfaceTension;
					Fluid="Air-Water";
					break;
				}
				case "SteamWater34.5bar":
				{
					SteamWater34barProperty FluidProperty=new SteamWater34barProperty();
					Rol=FluidProperty.LiquidDensity;
					Rog=FluidProperty.VapourDensity;
					Mul=FluidProperty.LiquidViscosity;
					Mug=FluidProperty.VapourViscosity;
					St=FluidProperty.SurfaceTension;
					Fluid="Steam-Water";
					break;
				}
				case "SteamWater69bar":
				{
					SteamWater69barProperty FluidProperty=new SteamWater69barProperty();
					Rol=FluidProperty.LiquidDensity;
					Rog=FluidProperty.VapourDensity;
					Mul=FluidProperty.LiquidViscosity;
					Mug=FluidProperty.VapourViscosity;
					St=FluidProperty.SurfaceTension;
					Fluid="Steam-Water";
					break;
				}
				case "R134a6bar":
				{
					R134a6barProperty FluidProperty=new R134a6barProperty();
					Rol=FluidProperty.LiquidDensity;
					Rog=FluidProperty.VapourDensity;
					Mul=FluidProperty.LiquidViscosity;
					Mug=FluidProperty.VapourViscosity;
					St=FluidProperty.SurfaceTension;
					Fluid="R134a";
					break;
				}
				case "R134a10bar":
				{
					R134a10barProperty FluidProperty=new R134a10barProperty();
					Rol=FluidProperty.LiquidDensity;
					Rog=FluidProperty.VapourDensity;
					Mul=FluidProperty.LiquidViscosity;
					Mug=FluidProperty.VapourViscosity;
					St=FluidProperty.SurfaceTension;
					Fluid="R134a";
					break;
				}
				case "R134a14bar":
				{
					R134a14barProperty FluidProperty=new R134a14barProperty();
					Rol=FluidProperty.LiquidDensity;
					Rog=FluidProperty.VapourDensity;
					Mul=FluidProperty.LiquidViscosity;
					Mug=FluidProperty.VapourViscosity;
					St=FluidProperty.SurfaceTension;
					Fluid="R134a";
					break;
				}
			}
		}

		private string getFlowState(double Um)
		{	
			double Re;
			string State;
			Re=Rol*Um*d/Mul;
			if (Re>=1502)
			{
				State="turbulent";
			}
			else
			{
				State="laminar";
			}
			return State;
		}

		private double getFrictionCoefficient(double Um)
		{	
			double Re,f;
			double c,n;
			Re=Rol*Um*d/Mul;
			if (Re>=1502)
			{
				c=0.046;
				n=0.2;
			}
			else
			{
				c=16;
				n=1;
			}
			f=c/Math.Pow(Re,n);
			return f;
		}
		private double getFrictionCoefficient(double Ugs,double Uls)
		{	
			double Um,Re,f;
			double c,n;
			Um=Ugs+Uls;
			Re=Rol*Um*d/Mul;
			if (Re>=1502)
			{
				c=0.046;
				n=0.2;
			}
			else
			{
				c=16;
				n=1;
			}
			f=c/Math.Pow(Re,n);
			return f;
		}

		private double getFrictionCoefficient(double Ugs,double Uls,out double c, out double n)
		{	
			double Um,Re,f;
			Um=Ugs+Uls;
			Re=0;
			if (Ugs==0)
			{
				Re=Rol*Um*d/Mul;
			}
			if (Uls==0)
			{
				Re=Rog*Um*d/Mug;
			}
			if (Re>=1502)
			{
				c=0.046;
				n=0.2;
			}
			else
			{
				c=16;
				n=1;
			}
			f=c/Math.Pow(Re,n);
			return f;
		}

		public void GetBubblyDispersedbubbleBoundary(out int StartPoint,out int EndPoint)
			// Barnea et al. (1982) - cited by Taitel (1990)
		{
			double voidFraction=0.0;
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double Um,Dst,Dcd,k;
			StartPoint=0;
			int i=StartPoint;
			do
			{
				X[i]=Math.Pow(10,(Math.Log10(MinUgs)+i*Xstep));
				if (i==StartPoint)
				{
					Y[i]=MinUls;
				}
				else 
				{
					Y[i]=Y[i-1];
				}
				do
				{
					Um=X[i]+Y[i];
					k=2*this.getFrictionCoefficient(X[i],Y[i])*Math.Pow(Um,3)/d;
					voidFraction=X[i]/(X[i]+Y[i]);
					Dst=(0.725+4.15*Math.Sqrt(voidFraction))*Math.Pow((St/Rol),0.6)*Math.Pow(k,-0.4);
					Dcd=2*Math.Sqrt(0.4*St/(g*(Rol-Rog)));
					Y[i]*=Dst/Dcd;
				} while (Math.Abs((Dst/Dcd)-1)>=0.001);
				BubblyDispersedbubbleBoundary[i].X=(float)X[i];
				BubblyDispersedbubbleBoundary[i].Y=(float)Y[i];
				i++;
			}while (voidFraction<=criticalVoidFraction1);
			EndPoint=i-1;
			maxIndexOfBubblyDispersedbubbleBoundary=EndPoint;
			maxUlsOfBubblyDispersedbubbleBoundary=Y[EndPoint];
		}

		public void GetDispersedbubbleChurnBoundary(out int StartPoint,out int EndPoint)
			// Taitel et al. (1990)
		{
			int startPoint,endPoint;
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			
			if (this.maxIndexOfBubblyDispersedbubbleBoundary==0)
			{
				this.GetBubblyDispersedbubbleBoundary(out startPoint,out endPoint);
				StartPoint=endPoint;
			}
			else 
			{
				StartPoint=this.maxIndexOfBubblyDispersedbubbleBoundary;
			}

			int i=StartPoint;
			do
			{
				X[i]=Math.Pow(10,(Math.Log10(MinUgs)+i*Xstep));
				Y[i]=(1-criticalVoidFraction1)*X[i]/criticalVoidFraction1;
				DispersedbubbleChurnBoundary[i].X=(float)X[i];
				DispersedbubbleChurnBoundary[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<=MaxUgs && Y[i-1]<=MaxUls);
			EndPoint=i-1;
		}

		public void GetIntermitentChurnBoundary(out int StartPoint,out int EndPoint)
			// Barnea & Brauner (1985) - cited by Taitel (1990)
		{
			int startPoint,endPoint;
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double Um,f,Dc,Dcd,liquidSlugVoidFraction;

			if (this.maxUlsOfBubblyDispersedbubbleBoundary==0)
			{
				this.GetBubblyDispersedbubbleBoundary(out startPoint,out endPoint);
			}

			StartPoint=0;
			int i=StartPoint;
			do
			{
				if (i==StartPoint)
				{
					X[i]=MinUgs;
				}
				else 
				{
					X[i]=X[i-1];
				}
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				do
				{
					Um=X[i]+Y[i];
					f=this.getFrictionCoefficient(X[i],Y[i]);
					Dcd=2*Math.Sqrt(0.4*St/(g*(Rol-Rog)));
					Dc=Dcd;
					liquidSlugVoidFraction=Math.Pow((2*f*Math.Pow(Um,3)/this.d),0.4);
					liquidSlugVoidFraction=liquidSlugVoidFraction*Math.Pow((Rol/St),0.6);
					liquidSlugVoidFraction=0.058*Math.Pow((Dc*liquidSlugVoidFraction-0.725),2);
					X[i]*=Math.Sqrt(criticalVoidFraction1/liquidSlugVoidFraction);
				} while (Math.Abs(liquidSlugVoidFraction/criticalVoidFraction1-1)>0.001);
				IntermitentChurnBoundary[i].X=(float)X[i];
				IntermitentChurnBoundary[i].Y=(float)Y[i];
				i++;
			}while (Y[i-1]<=maxUlsOfBubblyDispersedbubbleBoundary);
			EndPoint=i-1;
		}

		public void GetChurnAnnularBoundary1(out int StartPoint,out int EndPoint)
			// Mechanism A: Barnea (1986) - cited by Taitel (1990)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double Cl,n,Uls;
			double Dr,lastDr; //Liquid film thickness-tube diameter ratio
			double[] Co =new double[4];
			double surfaceFriction;
			double Cg,m,fg,fi,Tao;

			StartPoint=0;
			int i=StartPoint;
			Dr=0.001;

			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				this.getFrictionCoefficient(0,Y[i],out Cl,out n);
				do
				{
					Co[0]=Math.Pow(1-2*Dr,2)-2*(Dr-Math.Pow(Dr,2));
					Co[0]=g*(Rol-Rog)*d*Co[0];
					if (Co[0]<0)
					{
						break;
					}
					Co[1]=(Dr-Math.Pow(Dr,2)+Math.Pow(1-2*Dr,2))/Math.Pow(Dr-Math.Pow(Dr,2),3);
					Co[1]=Cl*Rol*Math.Pow(d*Rol/Mul,-1*n)*Co[1]/16;
					Uls=Math.Pow(Co[0]/Co[1],1/(2-n));
					Dr*=Math.Pow(Y[i]/Uls,0.3333);
					lastDr=Dr;
				} while (Math.Abs(Uls/Y[i]-1)>0.001);

				if (Co[0]<0) 
				{
					break;
				}

				Co[2]=(Dr-Math.Pow(Dr,2))*(1-2*Dr);
				Co[2]=g*(Rol-Rog)*d*Co[2];
				Co[3]=(1-2*Dr)/Math.Pow(Dr-Math.Pow(Dr,2),2);
				Co[3]=Cl*Rol*Math.Pow(d*Rol/Mul,-1*n)*Co[3]/32;
				surfaceFriction=Co[2]+Co[3]*Math.Pow(Y[i],2-n);

				if (i==StartPoint)
				{
					X[i]=MinUgs;
				}
				else 
				{
					X[i]=X[i-1];
				}

				do
				{
					fg=this.getFrictionCoefficient(X[i],0,out Cg,out m);
					fi=fg*(1+300*Dr);
					Tao=fi*Rog*Math.Pow(X[i],2)/(2*Math.Pow(1-2*Dr,4));
					X[i]*=Math.Pow(surfaceFriction/Tao,0.5);
				}while (Math.Abs(surfaceFriction/Tao-1)>0.001);
				ChurnAnnularBoundary1[i].X=(float)X[i];
				ChurnAnnularBoundary1[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
		}

		public void GetChurnAnnularBoundary2(out int StartPoint,out int EndPoint)
			// Mechanism B: Barnea (1986) - cited by Taitel (1990)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double Cl,n;
			double Dr; //Liquid film thickness-tube diameter ratio
			double[] Co =new double[4];
			double surfaceFriction;
			double Cg,m,fg,fi,Tao;

			StartPoint=0;
			int i=StartPoint;
			Dr=0.06411; //Barnea & Brauner, 1985, estimated the maximum stable liquid holdup is 0.48

			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				this.getFrictionCoefficient(0,Y[i],out Cl,out n);

				Co[2]=(Dr-Math.Pow(Dr,2))*(1-2*Dr);
				Co[2]=g*(Rol-Rog)*d*Co[2];
				Co[3]=(1-2*Dr)/Math.Pow(Dr-Math.Pow(Dr,2),2);
				Co[3]=Cl*Rol*Math.Pow(d*Rol/Mul,-1*n)*Co[3]/32;
				surfaceFriction=Co[2]+Co[3]*Math.Pow(Y[i],2-n);
				if (i==StartPoint)
				{
					X[i]=MinUgs;
				}
				else 
				{
					X[i]=X[i-1];
				}
				do
				{
					fg=this.getFrictionCoefficient(X[i],0,out Cg,out m);
					fi=fg*(1+300*Dr);
					Tao=fi*Rog*Math.Pow(X[i],2)/(2*Math.Pow(1-2*Dr,4));
					X[i]*=Math.Pow(surfaceFriction/Tao,0.5);
				}while (Math.Abs(surfaceFriction/Tao-1)>0.001);
				ChurnAnnularBoundary2[i].X=(float)X[i];
				ChurnAnnularBoundary2[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
		}

		public void GetBubblySlugBoundary(out int StartPoint,out int EndPoint)
			// Taitel (1980) - cited by Taitel (1990)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double U0=1.53*Math.Pow(g*St*(Rol-Rog)/(Rol*Rol),0.25);
			StartPoint=0;
			int i=StartPoint;
			int j=StartPoint;
			int k;
			double criticalD;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=criticalVoidFraction0*(Y[i]/(1-criticalVoidFraction0)+U0);
				BubblySlugBoundary[i].X=(float)X[i];
				BubblySlugBoundary[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryA1Valid)
			{
				for (k=0; k<1000; k++)
				{
					while (BubblyDispersedbubbleBoundary[j].X<=BubblySlugBoundary[k].X)
					{
						j++;
					}
					if (BubblyDispersedbubbleBoundary[j].Y<=BubblySlugBoundary[k].Y)
					{
						break;
					}
				}
				EndPoint=k;
			}
			criticalD=19.01*Math.Sqrt((Rol-Rog)*St/(g*Rol*Rol));
			if (d<criticalD)
			{
				EndPoint=StartPoint;
			}
		}

		public void GetChurnAnnularBoundaryB(out int StartPoint,out int EndPoint)
			// Taitel et al. (1980)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)

			StartPoint=0;
			int i=StartPoint;
			do
			{
				X[i]=3.1*Math.Pow((St*g*(Rol-Rog)/Math.Pow(Rog,2)),0.25);
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				ChurnAnnularBoundaryB[i].X=(float)X[i];
				ChurnAnnularBoundaryB[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
		}

		public void GetDispersedbubbleBubblyBoundaryB(out int StartPoint,out int EndPoint)
			// Taitel et al. (1980)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)

			StartPoint=0;
			int i=StartPoint;
			int k;
			int j=0;
			do
			{
				X[i]=Math.Pow(10,(Math.Log10(MinUgs)+i*Xstep));
				Y[i]=4.0*(Math.Pow(d,0.429)*Math.Pow(St/Rol,0.089)/Math.Pow(Mul/Rol,0.072))*Math.Pow(g*(Rol-Rog)/Rol,0.446)-X[i];
				DispersedbubbleBubblyBoundaryB[i].X=(float)X[i];
				DispersedbubbleBubblyBoundaryB[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls && Y[i-1]>MinUls);
			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryB3Valid)
			{
				for (k=0; k<1000; k++)
				{
					while (DispersedbubbleChurnBoundaryB[j].X<=DispersedbubbleBubblyBoundaryB[k].X)
					{
						j++;
					}
					if (DispersedbubbleChurnBoundaryB[j].Y>=DispersedbubbleBubblyBoundaryB[k].Y)
					{
						break;
					}
				}
				EndPoint=k;
			}
		}

		public void GetDispersedbubbleChurnBoundaryB(out int StartPoint,out int EndPoint)
			// Taitel et al. (1980)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)

			StartPoint=0;
			int i=StartPoint;
			int k, m;
			int j=0;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=criticalVoidFraction1*Y[i]/(1-criticalVoidFraction1);
				DispersedbubbleChurnBoundaryB[i].X=(float)X[i];
				DispersedbubbleChurnBoundaryB[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryB2Valid)
			{
				for (k=0; k<1000; k++)
				{
					while (DispersedbubbleBubblyBoundaryB[j].X<=DispersedbubbleChurnBoundaryB[k].X)
					{
						j++;
					}
					if (DispersedbubbleBubblyBoundaryB[j].Y<=DispersedbubbleChurnBoundaryB[k].Y)
					{
						break;
					}
				}
				StartPoint=k;
				dispersedbubbleChurnBoundaryStartPoint=k;
			}
			if (FrmFlowPatternGraph.isBoundaryB1Valid)
			{
				for (m=0; m<1000; m++)
				{
					if (DispersedbubbleChurnBoundaryB[m].X>ChurnAnnularBoundaryB[0].X)
					{
						break;
					}
				}
				EndPoint=m;
				if (EndPoint>999)
				{
					EndPoint=998;
				}
			}
		}

		public void GetBubblySlugBoundaryB(out int StartPoint,out int EndPoint)
			// Taitel et al. (1980)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double U0=1.53*Math.Pow(g*St*(Rol-Rog)/(Rol*Rol),0.25);
			StartPoint=0;
			int i=StartPoint;
			int j=StartPoint;
			int k;
			double criticalD;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=criticalVoidFraction0*(Y[i]/(1-criticalVoidFraction0)+U0);
				BubblySlugBoundaryB[i].X=(float)X[i];
				BubblySlugBoundaryB[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryB2Valid)
			{
				for (k=0; k<1000; k++)
				{
					while (DispersedbubbleBubblyBoundaryB[j].X<=BubblySlugBoundaryB[k].X)
					{
						j++;
					}
					if (DispersedbubbleBubblyBoundaryB[j].Y<=BubblySlugBoundaryB[k].Y)
					{
						break;
					}
				}
				EndPoint=k;
			}
			criticalD=19.01*Math.Sqrt((Rol-Rog)*St/(g*Rol*Rol));
			if (d<criticalD)
			{
				EndPoint=StartPoint;
			}		
		}

		public void GetSlugChurnBoundaryB(out int StartPoint,out int EndPoint)
			// Taitel et al. (1980)
		{
			double[] X1=new double[GridNumber]; //Vapour superficial velocity from Slug-Churn transition model equation(m/s)
			double[] X2=new double[GridNumber]; //Vapour superficial velocity from Bubble-Slug transition model equation (m/s)
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double U0=1.53*Math.Pow(g*St*(Rol-Rog)/(Rol*Rol),0.25);
			StartPoint=0;
			int i=StartPoint;
			int j=StartPoint;
			int k=StartPoint;
			int m=StartPoint;
			int n=StartPoint;
			int l=StartPoint;
			int o=StartPoint;
			int q=StartPoint;
			double LDratio=0;
			double ID=Math.Round(1000*100*d);
			if (ID==110)
			{
				LDratio=LDratio110;
			}
			if (ID==201)
			{
				LDratio=LDratio201;
			}

			if (ID==288)
			{
				LDratio=LDratio288;
			}

			if (ID==426)
			{
				LDratio=LDratio426;
			}
			else
			{
				LDratio=LDratioDefault;
			}

			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X1[i]=(LDratio/40.6-0.22)*Math.Sqrt(g*d)-Y[i];
				X2[i]=criticalVoidFraction0*(Y[i]/(1-criticalVoidFraction0)+U0);
				X[i]=Math.Max(X1[i],X2[i]);
				SlugChurnBoundaryB[i].X=(float)X[i];
				SlugChurnBoundaryB[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls && X[i-1]>MinUgs);
			EndPoint=i-1;

			if (FrmFlowPatternGraph.isBoundaryB1Valid)
			{
				for (l=0; l<1000; l++)
				{
					if (SlugChurnBoundaryB[l].X<=ChurnAnnularBoundaryB[0].X)
					{
						break;
					}
				}
				StartPoint=l;
			}

			if (FrmFlowPatternGraph.isBoundaryB3Valid)
			{
				for (m=0; m<1000; m++)
				{
					while (DispersedbubbleChurnBoundaryB[n].Y<=SlugChurnBoundaryB[m].Y)
					{
						n++;
					}
					if (DispersedbubbleChurnBoundaryB[n].X>=SlugChurnBoundaryB[m].X)
					{
						break;
					}
				}
				EndPoint=m;
			}

			if ((n<dispersedbubbleChurnBoundaryStartPoint) && FrmFlowPatternGraph.isBoundaryB4Valid)
			{
				for (k=0; k<1000; k++)
				{
					while (BubblySlugBoundaryB[j].Y<=SlugChurnBoundaryB[k].Y)
					{
						j++;
					}
					if (BubblySlugBoundaryB[j].X>=SlugChurnBoundaryB[k].X)
					{
						break;
					}
				}
				EndPoint=k;
			}

			if ((n<dispersedbubbleChurnBoundaryStartPoint) && FrmFlowPatternGraph.isBoundaryB2Valid)
			{
				for (o=0; o<1000; o++)
				{
					while (DispersedbubbleBubblyBoundaryB[q].Y>=SlugChurnBoundaryB[o].Y)
					{
						if (q<998)
						{
							q++;
						}
						else
						{
							break;
						}
					}
					if (DispersedbubbleBubblyBoundaryB[q].X<=SlugChurnBoundaryB[o].X)
					{
						break;
					}
					q=0;
				}
				EndPoint=o;
			}

		}

		public void GetBubblySlugBoundaryC(out int StartPoint,out int EndPoint)
			// Mishima & Ishii. (1984)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double C0=1.2-0.2*Math.Sqrt(Rog/Rol);
			double C1=Math.Sqrt(2.0)*Math.Pow(St*g*(Rol-Rog)/(Rol*Rol),0.25)*Math.Pow((1-criticalVoidFraction2),1.75)/C0;
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=(Y[i]+C1)/(1/(criticalVoidFraction2*C0)-1);
				BubblySlugBoundaryC[i].X=(float)X[i];
				BubblySlugBoundaryC[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
		}

		public void GetSlugChurnBoundaryC(out int StartPoint,out int EndPoint)
			// Mishima & Ishii. (1984)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double C0=1.2-0.2*Math.Sqrt(Rog/Rol);
			double C1=0.35*Math.Sqrt((Rol-Rog)*g*d/Rol);
			double C2=0.75*Math.Sqrt((Rol-Rog)*g*d/Rol)*Math.Pow((Rol-Rog)*g*d*d*d/(Rol*Math.Pow(Mul/Rol,2)),1.0/18.0);
			double Us=10; //Us=Ugs+Uls
			double Uls=MinUls;
			StartPoint=0;
			int i=StartPoint;
			int k=StartPoint;
			do
			{
				Us=Us*Math.Pow(MinUls/Uls,0.001);
				Uls=Us-(C0*Us+C1)*(1-0.813*Math.Pow(((C0-1)*Us+C1)/(Us+C2),0.75));
			}while (Math.Abs(Uls/MinUls-1)>0.001);
			do
			{
				Us=Math.Pow(10,(Math.Log10(Us)+i*(Ystep+Xstep)/10));
				Y[i]=Us-(C0*Us+C1)*(1-0.813*Math.Pow(((C0-1)*Us+C1)/(Us+C2),0.75));
				X[i]=Us-Y[i];
				SlugChurnBoundaryC[i].X=(float)X[i];
				SlugChurnBoundaryC[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);

			if (FrmFlowPatternGraph.isBoundaryC3Valid)
			{
				for (k=0; k<1000; k++)
				{
					if (SlugChurnBoundaryC[k].X>=SlugAnnularBoundaryC[0].X)
					{
						break;
					}
				}
				EndPoint=k;
			}
			else
			{
				EndPoint=i-1;
			}
		}

		public void GetSlugAnnularBoundaryC(out int StartPoint,out int EndPoint)
			// Mishima & Ishii. (1984)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			
			double Nuf=Mul/Math.Sqrt(Rol*St*Math.Sqrt(St/(g*(Rol-Rog))));
			double Ulg=Math.Pow(St*g*(Rol-Rog)/(Rog*Rog),0.25)*Math.Pow(Nuf,-0.2);
			StartPoint=0;
			int i=StartPoint;
			int j=StartPoint;
			int n=StartPoint;
			int k,m;
			do
			{
				X[i]=Ulg;
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				SlugAnnularBoundaryC[i].X=(float)X[i];
				SlugAnnularBoundaryC[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryC2Valid)
			{
				for (k=0; k<1000; k++)
				{
					while (SlugChurnBoundaryC[j].Y<=SlugAnnularBoundaryC[k].Y)
					{
						j++;
					}
					if (SlugChurnBoundaryC[j].X>=SlugAnnularBoundaryC[k].X)
					{
						break;
					}
				}
				StartPoint=k;
			}
			if (FrmFlowPatternGraph.isBoundaryC1Valid)
			{
				for (m=0; m<1000; m++)
				{
					while (BubblySlugBoundaryC[n].Y<=SlugAnnularBoundaryC[m].Y)
					{
						n++;
					}
					if (BubblySlugBoundaryC[n].X>=SlugAnnularBoundaryC[m].X)
					{
						break;
					}
				}
				EndPoint=m;
			}
		}

		public void GetChurnAnnularBoundaryC(out int StartPoint,out int EndPoint)
			// Mishima & Ishii. (1984)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double C0=1.2-0.2*Math.Sqrt(Rog/Rol);
			double C1=0.35*Math.Sqrt((Rol-Rog)*g*d/Rol);
			double C2=0.75*Math.Sqrt((Rol-Rog)*g*d/Rol)*Math.Pow((Rol-Rog)*g*d*d*d/(Rol*Math.Pow(Mul/Rol,2)),1.0/18.0);
			double Us=1; //Us=Ugs+Uls
			double alfa1,alfa2;
			StartPoint=0;
			int i=StartPoint;
			int j=StartPoint;
			int k;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				do
				{
					alfa1=(Us-Y[i])*(Us-Y[i])/((Rol-Rog)*g*d/Rog)+0.11;
					alfa2=1-0.813*Math.Pow(((C0-1)*Us+C1)/(Us+C2),0.75);
					Us=Us*Math.Pow(alfa2/alfa1,0.01);
				} while (Math.Abs(alfa1/alfa2-1)>0.001);
				X[i]=Us-Y[i];
				ChurnAnnularBoundaryC[i].X=(float)X[i];
				ChurnAnnularBoundaryC[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryC2Valid)
			{
				for (k=0; k<1000; k++)
				{
					while (SlugChurnBoundaryC[j].Y<=ChurnAnnularBoundaryC[k].Y)
					{
						j++;
					}
					if (SlugChurnBoundaryC[j].X>=ChurnAnnularBoundaryC[k].X)
					{
						break;
					}
				}
				EndPoint=k;
			}
		}

		public void GetBubblyPlugChurnBoundaryD(out int StartPoint,out int EndPoint)
			// McQuillan & Whalley (1985)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double C=1.15*Math.Pow(g*St*(Rol-Rog)/(Rol*Rol),0.25);
			StartPoint=0;
			int i=StartPoint;
			int k=StartPoint;
			int j=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=(Y[i]+C)/3.0;
				BubblyPlugChurnBoundaryD[i].X=(float)X[i];
				BubblyPlugChurnBoundaryD[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);

			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryD3Valid)
			{
				for (k=0; k<1000; k++)
				{
					if (BubblyPlugChurnBoundaryD[k].Y>FromDispersedbubbleBoundaryD[0].Y)
					{
						break;
					}
				}
				EndPoint=k;
			}
			if (FrmFlowPatternGraph.isBoundaryD5Valid)
			{
				for (j=0; j<1000; j++)
				{
					if (BubblyPlugChurnBoundaryD[j].X>FromAnnularBoundaryD[0].X)
					{
						break;
					}
				}
				EndPoint=j;
			}
			if (FrmFlowPatternGraph.isBoundaryD3Valid && FrmFlowPatternGraph.isBoundaryD5Valid)
			{
				EndPoint=Math.Min(k,j);
			}
		}

		public void GetDispersedbubbleChurnBoundaryD(out int StartPoint,out int EndPoint)
			// McQuillan & Whalley (1985)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			StartPoint=0;
			int i=StartPoint;
			int j,k;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=Y[i]*criticalVoidFraction3/(1-criticalVoidFraction3);
				DispersedbubbleChurnBoundaryD[i].X=(float)X[i];
				DispersedbubbleChurnBoundaryD[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;

			if (FrmFlowPatternGraph.isBoundaryD3Valid)
			{
				for (k=0; k<1000; k++)
				{
					if (DispersedbubbleChurnBoundaryD[k].Y>FromDispersedbubbleBoundaryD[0].Y)
					{
						break;
					}
				}
				StartPoint=k;
			}
			if (FrmFlowPatternGraph.isBoundaryD5Valid)
			{
				for (j=0; j<1000; j++)
				{
					if (DispersedbubbleChurnBoundaryD[j].X>FromAnnularBoundaryD[0].X)
					{
						break;
					}
				}
				EndPoint=j;
			}
		}

		public void GetFromDispersedbubbleBoundaryD(out int StartPoint,out int EndPoint)
			// McQuillan & Whalley (1985)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			StartPoint=0;
			int i=StartPoint;
			int j,k,l,m,n;
			double lEdge,nEdge;
			do
			{
				X[i]=Math.Pow(10,(Math.Log10(MinUgs)+i*Xstep));
				Y[i]=6.8*Math.Pow(g*St*(Rol-Rog),0.278)*Math.Pow(d/Mul,0.112)/Math.Pow(Rol,0.444);
				FromDispersedbubbleBoundaryD[i].X=(float)X[i];
				FromDispersedbubbleBoundaryD[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryD1Valid)
			{
				for (j=0; j<1000; j++)
				{
					if (BubblyPlugChurnBoundaryD[j].Y>FromDispersedbubbleBoundaryD[0].Y)
					{
						break;
					}
				}
				lEdge=Math.Log10(BubblyPlugChurnBoundaryD[j].X/MinUgs)/Xstep;
				for (l=0; l<1000; l++)
				{
					if (l>lEdge)
					{
						break;
					}
				}
				StartPoint=l;
			}		
			if (FrmFlowPatternGraph.isBoundaryD5Valid)
			{
				for (k=0; k<1000; k++)
				{
					if (FromDispersedbubbleBoundaryD[k].X>FromAnnularBoundaryD[0].X)
					{
						break;
					}
				}
				EndPoint=k;
			}	
			if (FrmFlowPatternGraph.isBoundaryD2Valid)
			{
				for (m=0; m<1000; m++)
				{
					if (DispersedbubbleChurnBoundaryD[m].Y>FromDispersedbubbleBoundaryD[0].Y)
					{
						break;
					}
				}
				nEdge=(Math.Log10(DispersedbubbleChurnBoundaryD[m].X/MinUgs)/Xstep);
				for (n=0; n<1000; n++)
				{
					if (n>nEdge)
					{
						break;
					}
				}
				EndPoint=Math.Min(EndPoint,n);
			}		
		}

		public void GetPlugChurnBoundaryD(out int StartPoint,out int EndPoint)
			// McQuillan & Whalley (1985)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double A=Math.PI*d*d/4;
			double c,filmThichness;
			double c1=Math.Sqrt(Rog)/(A*Math.Sqrt(g*d*(Rol-Rog)));
			double c2=Math.Sqrt(Rol)/(A*Math.Sqrt(g*d*(Rol-Rog)));
			double c3=0.35*A*Math.Sqrt(g*d*(Rol-Rog)/Rol);
			double Qp, Qf=0;
			double function;
			StartPoint=0;
			int i=StartPoint;
			int j=StartPoint;
			int k;
			do
			{
				Qf=Qf+MinUls*A;
				Qp=Math.Pow((plugChurnTransitionConstant-Math.Sqrt(Qf*c2)),2)/c1;
				filmThichness=Math.Pow((3*Qf*Mul/(Math.PI*g*d*Rol)),1.0/3.0);
				c=1-4*filmThichness/d;
				function=Qp-c*(1.2*(Qp-Qf)+c3);
			}while (function<0);
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=(Qp-Qf)/A-Y[i];
				PlugChurnBoundaryD[i].X=(float)X[i];
				PlugChurnBoundaryD[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls && X[i-1]>MinUgs);
			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryD1Valid)
			{
				for (k=0; k<1000; k++)
				{
					while (BubblyPlugChurnBoundaryD[j].Y<=PlugChurnBoundaryD[k].Y)
					{
						j++;
					}
					if (BubblyPlugChurnBoundaryD[j].X>=PlugChurnBoundaryD[k].X)
					{
						break;
					}
				}
				EndPoint=k;
			}
		}

		public void GetFromAnnularBoundaryD(out int StartPoint,out int EndPoint)
			// McQuillan & Whalley (1985)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=criticalDimensionlessUgs*Math.Sqrt(g*d*(Rol-Rog))/Math.Sqrt(Rog);
				FromAnnularBoundaryD[i].X=(float)X[i];
				FromAnnularBoundaryD[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
		}

		public void GetBubblySlugBoundaryE(out int StartPoint,out int EndPoint)
			// Chen et al. (2006)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double[] Uh=new double[GridNumber]; //homogeneous velocity (m/s)
			double Ud=0.0;	//bubble drift velocity (m/s)
			double criticalVoidFraction;
			double c1=0.50*Mul/Mulref;
			double c2=650.0*Math.Sqrt(St/Stref);
			double C0=1.2-0.2*Math.Sqrt(Rog/Rol)+c1*Math.Exp(-c2*d);
			double C1=0.0;
			double C2=0.0;
			switch (Fluid)
			{
				case "R134a":
				{
					C1=0.138;
					C2=0.344;
					if (Math.Abs(d-0.00110)<0.00001)	Ud=0.006;
					if (Math.Abs(d-0.00201)<0.00001)	Ud=0.012;
					if (Math.Abs(d-0.00288)<0.00001)	Ud=0.025;
					if (Math.Abs(d-0.00426)<0.00001)	Ud=0.048;
					break;
				}
				case "Air-Water":
				{
					C1=0.138;
					C2=0.344;
					if (d<0.0056)
					{
						Ud=0.0;
					}
					else 
					{
						Ud=0.35*Math.Sqrt((Rol-Rog)*g*d/Rol);
					}
					break;
				}
			}
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Uh[i]=Math.Pow(10,(Math.Log10(MinUgs)+i*Xstep))+Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				criticalVoidFraction=C1*Math.Pow(Uh[i],C2);
				X[i]=criticalVoidFraction*(C0*Uh[i]+Ud);
				Y[i]=Uh[i]-X[i];
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				BubblySlugBoundaryE[i].X=(float)X[i];
				BubblySlugBoundaryE[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;

			int l=0, j=0;
			if (FrmFlowPatternGraph.isBoundaryE4Valid)
			{
				for (l=0; l<1000; l++)
				{
					while (BubblySlugBoundaryE[j].Y<=ChurnAnnularBoundaryE1[l].Y)
					{
						j++;
					}
					if (BubblySlugBoundaryE[j].X>=ChurnAnnularBoundaryE1[l].X)
					{
						break;
					}
				}
				EndPoint=j;
			}
		}

		public void GetDispersedbubbleBubblyBoundaryE(out int StartPoint,out int EndPoint)
			// Chen et al. (2006)
		{
			double[] X=new double[GridNumber];	//Nominal vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber];	//Nominal liquid superficial velocity (m/s)
			double[] Ugs=new double[GridNumber];	//actual vapour superficial velocity (m/s)
			double[] Uls=new double[GridNumber];	//actual liquid superficial velocity (m/s)
			double Um,Uh;						//homogeneous velocity (m/s)
			double dmin,dcrit;				    //diameter (m)
			double actualVoidFraction;
			double We,f,a,b,c;
			double k=1.0;						//void fraction correction coefficient
			double A=0.45;
			double B=4.0;
			double m=1.0/2.0;
			double n=1.0/3.0;
			dmin=2.0*Math.Sqrt(0.4*St/((Rol-Rog)*g));
			dcrit=0.5*d;
			switch (Fluid)
			{
				case "R134a":
				{
					if (Math.Abs(d-0.00110)<0.00001)	
					{
						if (Math.Abs(p-600000)<50000)	
						{
							dcrit=0.46/1000.0;
							k=0.212;
						}
						if (Math.Abs(p-1000000)<50000)
						{
							dcrit=0.43/1000.0;
							k=0.240;
						}
						if (Math.Abs(p-1400000)<50000)
						{
							dcrit=0.45/1000.0;
							k=0.296;
						}
					}
					if (Math.Abs(d-0.00201)<0.00001)
					{
						if (Math.Abs(p-600000)<50000)
						{
							dcrit=0.95/1000.0;
							k=0.400;
						}
						if (Math.Abs(p-1000000)<50000)
						{
							dcrit=1.0/1000.0;
							k=0.604;
						}
						if (Math.Abs(p-1400000)<50000)
						{
							dcrit=0.92/1000.0;
							k=0.625;
						}
					}
					if (Math.Abs(d-0.00288)<0.00001)	
					{
						if (Math.Abs(p-600000)<50000)	
						{
							dcrit=0.87/1000.0;
							k=0.181;
						}
						if (Math.Abs(p-1000000)<50000)
						{
							dcrit=0.98/1000.0;
							k=0.371;
						}
						if (Math.Abs(p-1400000)<50000)
						{
							dcrit=1.03/1000.0;
							k=0.624;
						}
					}
					if (Math.Abs(d-0.00426)<0.00001)
					{
						if (Math.Abs(p-600000)<50000)	
						{
							dcrit=1.05/1000.0;
							k=0.299;
						}
						if (Math.Abs(p-1000000)<50000)	
						{
							dcrit=1.10/1000.0;
							k=0.391;
						}
						if (Math.Abs(p-1400000)<50000)	
						{
							dcrit=1.18/1000.0;
							k=0.562;
						}
					}
					break;
				}
				case "Air-Water":
				{
					dcrit=Math.Min(d,dmin);
					break;
				}
			}
			StartPoint=0;
			int i=StartPoint;
			Uh=1.0;
			do
			{
				X[i]=Math.Pow(10,(Math.Log10(MinUgs)+i*Xstep));
				do
				{
					Um=Uh;
					a=Rog-Rol;
					b=Rol*Um+(Rol-Rog)*X[i];
					c=-k*Rol*Um*X[i];
					Ugs[i]=(-b+Math.Sqrt(b*b-4.0*a*c))/(2.0*a);
					actualVoidFraction=Ugs[i]/Um;
					f=this.getFrictionCoefficient(Um);
					We=Rog*dcrit*Um*Um/St;
					Uh=A*(1.0+B*Math.Pow(actualVoidFraction*Math.Pow(We,m),n))*Math.Sqrt(St/(f*Rol*dcrit));
				}while (Math.Abs(Um/Uh-1)>0.001);
				Uls[i]=Uh-Ugs[i];
				Y[i]=k*X[i]*Uh/Ugs[i]-X[i];
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				DispersedbubbleBubblyBoundaryE[i].X=(float)X[i];
				DispersedbubbleBubblyBoundaryE[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls && Y[i-1]>MinUls);
			EndPoint=i-2;
			
			int l=0, j=0;
			if (FrmFlowPatternGraph.isBoundaryE1Valid)
			{
				for (l=0; l<1000; l++)
				{
					while (DispersedbubbleBubblyBoundaryE[j].X<=BubblySlugBoundaryE[l].X)
					{
						j++;
					}
					if (DispersedbubbleBubblyBoundaryE[j].Y<=BubblySlugBoundaryE[l].Y)
					{
						break;
					}
				}
				EndPoint=j;
				if (EndPoint>999)
				{
					EndPoint=998;
				}
			}
		}

		public void GetDispersedbubbleChurnBoundaryE(out int StartPoint,out int EndPoint)
			// Chen. (2006)
		{
			//StartPoint=0;
			//EndPoint=0;
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double[] Uh=new double[GridNumber]; //homogeneous velocity (m/s)
			double Ud=0.0;	//bubble drift velocity (m/s)
			double criticalVoidFraction;
			double C0=1.0;
			double C1=0.138;
			double C2=0.344;
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Uh[i]=Math.Pow(10,(Math.Log10(MinUgs)+i*Xstep))+Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				criticalVoidFraction=C1*Math.Pow(Uh[i],C2);
				X[i]=criticalVoidFraction*(C0*Uh[i]+Ud);
				Y[i]=Uh[i]-X[i];
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				DispersedbubbleBubblyBoundaryE[i].X=(float)X[i];
				DispersedbubbleBubblyBoundaryE[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
		}

		public void GetSlugChurnBoundaryE1(out int StartPoint,out int EndPoint)
			//High Uls region, Chen et al. (2006)
		{
			double[] X=new double[GridNumber];	//Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber];	//Liquid superficial velocity (m/s)
			double Um,Uh,f;						//homogeneous velocity (m/s)
			StartPoint=0;
			int i=StartPoint;
			Uh=1.0;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				do
				{
					Um=Uh;
					f=this.getFrictionCoefficient(Um);
					Uh=2.75*Math.Sqrt(St/(f*Rol*d));
				}while (Math.Abs(Um/Uh-1)>0.001);
				X[i]=Uh-Y[i];
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				SlugChurnBoundaryE1[i].X=(float)X[i];
				SlugChurnBoundaryE1[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls && X[i-1]>MinUgs);
			EndPoint=i-2;

			int l1, j1=0;
			if (FrmFlowPatternGraph.isBoundaryE7Valid)
			{
				for (l1=0; l1<1000; l1++)
				{
					while (SlugChurnBoundaryE1[j1].Y<=SlugChurnBoundaryE2[l1].Y)
					{
						j1++;
					}
					if (SlugChurnBoundaryE1[j1].X<=SlugChurnBoundaryE2[l1].X)
					{
						break;
					}
				}
				StartPoint=j1;
			}
			int l2, j2=0;
			if (FrmFlowPatternGraph.isBoundaryE8Valid)
			{
				for (l2=0; l2<1000; l2++)
				{
					while (SlugChurnBoundaryE1[j2].Y<=SlugChurnBoundaryE3[l2].Y)
					{
						j2++;
					}
					if (SlugChurnBoundaryE1[j2].X<=SlugChurnBoundaryE3[l2].X)
					{
						break;
					}
				}
				StartPoint=j2;
			}
			StartPoint=Math.Max(j1,j2);

			int l3, j3=0;
			if (FrmFlowPatternGraph.isBoundaryE1Valid)
			{
				for (l3=0; l3<1000; l3++)
				{
					while (SlugChurnBoundaryE1[j3].Y<=BubblySlugBoundaryE[l3].Y)
					{
						j3++;
					}
					if (SlugChurnBoundaryE1[j3].X<=BubblySlugBoundaryE[l3].X)
					{
						break;
					}
				}
				EndPoint=j3;
			}
		}

		public void GetSlugChurnBoundaryE2(out int StartPoint,out int EndPoint)
			//Middle Uls region, Chen et al. (2006)
		{
			double[] X=new double[GridNumber];	//Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber];	//Liquid superficial velocity (m/s)
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=Math.Pow(Mul*Math.Pow(St,-0.6523)*Math.Pow(10,1.927),1.0/(1-0.6523));
				X[i]=Math.Sqrt(X[i]/(Rog*d));
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				SlugChurnBoundaryE2[i].X=(float)X[i];
				SlugChurnBoundaryE2[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-2;

			int l1, j1=0;
			if (FrmFlowPatternGraph.isBoundaryE3Valid)
			{
				for (l1=0; l1<1000; l1++)
				{
					while (SlugChurnBoundaryE2[j1].Y<=SlugChurnBoundaryE1[l1].Y)
					{
						j1++;
					}
					if (SlugChurnBoundaryE2[j1].X>=SlugChurnBoundaryE1[l1].X)
					{
						break;
					}
				}
				EndPoint=j1;
			}
			int l2, j2=0;
			if (FrmFlowPatternGraph.isBoundaryE8Valid)
			{
				for (l2=0; l2<1000; l2++)
				{
					while (SlugChurnBoundaryE2[j2].Y<=SlugChurnBoundaryE3[l2].Y)
					{
						j2++;
					}
					if (SlugChurnBoundaryE2[j2].X<=SlugChurnBoundaryE3[l2].X)
					{
						break;
					}
				}
				StartPoint=j2;
			}
			if (StartPoint>EndPoint) StartPoint=EndPoint;
		}

		public void GetSlugChurnBoundaryE3(out int StartPoint,out int EndPoint)
			//Middle Uls region, Chen et al. (2006)
		{
			double[] X=new double[GridNumber];	//Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber];	//Liquid superficial velocity (m/s)
			double Rels;
			double CFrgs,CWegs;
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				Rels=Rol*d*Y[i]/Mul;
				CFrgs=Math.Pow(Rog/((Rol-Rog)*g*d),-0.267/2.0);
				CWegs=Math.Pow(Rog*d/St,1.626);
				X[i]=Math.Pow(Rels/(CFrgs*CWegs*Math.Pow(10,1.9089)),1.0/(2*1.626-0.267));
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				SlugChurnBoundaryE3[i].X=(float)X[i];
				SlugChurnBoundaryE3[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-2;

			int l1, j1=0;
			if (FrmFlowPatternGraph.isBoundaryE7Valid)
			{
				for (l1=0; l1<1000; l1++)
				{
					while (SlugChurnBoundaryE3[j1].Y<=SlugChurnBoundaryE2[l1].Y)
					{
						j1++;
					}
					if (SlugChurnBoundaryE3[j1].X>=SlugChurnBoundaryE2[l1].X)
					{
						break;
					}
				}
				EndPoint=j1;
			}
			else
			{
				j1=1000;
			}
			int l2, j2=0;
			if (FrmFlowPatternGraph.isBoundaryE3Valid)
			{
				for (l2=0; l2<1000; l2++)
				{
					while (SlugChurnBoundaryE3[j2].Y<=SlugChurnBoundaryE1[l2].Y)
					{
						j2++;
					}
					if (SlugChurnBoundaryE3[j2].X>=SlugChurnBoundaryE1[l2].X)
					{
						break;
					}
				}
				EndPoint=j2;
			}
			else
			{
				j2=1000;
			}
			EndPoint=Math.Min(j1,j2);
		}

		public void GetChurnAnnularBoundaryE1(out int StartPoint,out int EndPoint)
			//High Ugs and Uls region, Chen et al. (2006)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=Math.Sqrt(Math.Pow(10,5.494)*Math.Sqrt(g/d)*Mug/Rog);
				ChurnAnnularBoundaryE1[i].X=(float)X[i];
				ChurnAnnularBoundaryE1[i].Y=(float)Y[i];
				i++;
			}while (Y[i-1]<MaxUls);
			EndPoint=i-1;

			int l=0, j=0;
			if (FrmFlowPatternGraph.isBoundaryE6Valid)
			{
				for (l=0; l<1000; l++)
				{
					while (ChurnAnnularBoundaryE1[j].Y<=ChurnAnnularBoundaryE2[l].Y)
					{
						j++;
					}
					if (ChurnAnnularBoundaryE1[j].X<=ChurnAnnularBoundaryE2[l].X)
					{
						break;
					}
				}
				StartPoint=j;
			}
		}

		public void GetChurnAnnularBoundaryE2(out int StartPoint,out int EndPoint)
			// Low Ugs and Uls region, Chen et al. (2006)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double temp;
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				temp=(16.805+Math.Log10(Rol*d*Y[i]*Y[i]/St))/3.4094;
				X[i]=Math.Sqrt(Math.Pow(10,temp)*Math.Sqrt(g/d)*Mug/Rog);
				ChurnAnnularBoundaryE2[i].X=(float)X[i];
				ChurnAnnularBoundaryE2[i].Y=(float)Y[i];
				i++;
			}while (Y[i-1]<MaxUls);
			EndPoint=i-1;

			int j=0;
			if (FrmFlowPatternGraph.isBoundaryE4Valid)
			{
				while (ChurnAnnularBoundaryE1[0].X>ChurnAnnularBoundaryE2[j].X)
				{
					j++;
				}
			EndPoint=j;
			}
			if (EndPoint>999)
			{
				EndPoint=998;
			}
		}

		public void GetBubblySlugBoundaryF(out int StartPoint,out int EndPoint)
			// Chen et al. (2006)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			double U0=1.53*Math.Pow(g*St*(Rol-Rog)/(Rol*Rol),0.25);
			StartPoint=0;
			int i=StartPoint;
			int j=StartPoint;
			int k;
			double criticalD;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=criticalVoidFraction0*(Y[i]/(1-criticalVoidFraction0)+U0);
				BubblySlugBoundaryF[i].X=(float)X[i];
				BubblySlugBoundaryF[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryF2Valid)
			{
				for (k=0; k<1000; k++)
				{
					while (DispersedbubbleBubblyBoundaryF[j].X<=BubblySlugBoundaryF[k].X)
					{
						j++;
					}
					if (DispersedbubbleBubblyBoundaryF[j].Y<=BubblySlugBoundaryF[k].Y)
					{
						break;
					}
				}
				EndPoint=k;
			}
			criticalD=19.01*Math.Sqrt((Rol-Rog)*St/(g*Rol*Rol))-0.006;
			if (d<criticalD)
			{
				EndPoint=StartPoint;
			}	
		}

		public void GetDispersedbubbleBubblyBoundaryF(out int StartPoint,out int EndPoint)
			// Chen et al. (2006)
		{
			double[] X=new double[GridNumber];	//Nominal vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber];	//Nominal liquid superficial velocity (m/s)
			double[] Ugs=new double[GridNumber];	//actual vapour superficial velocity (m/s)
			double[] Uls=new double[GridNumber];	//actual liquid superficial velocity (m/s)
			double Um,Uh;						//homogeneous velocity (m/s)
			double dmin,dcrit;				    //diameter (m)
			double actualVoidFraction;
			double We,f,a,b,c;
			double k=1.0;						//void fraction correction coefficient
			double A=0.45;
			double B=4.0;
			double m=1.0/2.0;
			double n=1.0/3.0;
			dmin=2.0*Math.Sqrt(0.4*St/((Rol-Rog)*g));
			dcrit=0.5*d;
			switch (Fluid)
			{
				case "R134a":
				{
					if (Math.Abs(d-0.00110)<0.00001)	
					{
						if (Math.Abs(p-600000)<50000)	
						{
							dcrit=0.46/1000.0;
							k=0.212;
						}
						if (Math.Abs(p-1000000)<50000)
						{
							dcrit=0.43/1000.0;
							k=0.240;
						}
						if (Math.Abs(p-1400000)<50000)
						{
							dcrit=0.45/1000.0;
							k=0.296;
						}
					}
					if (Math.Abs(d-0.00201)<0.00001)
					{
						if (Math.Abs(p-600000)<50000)
						{
							dcrit=0.95/1000.0;
							k=0.400;
						}
						if (Math.Abs(p-1000000)<50000)
						{
							dcrit=1.0/1000.0;
							k=0.604;
						}
						if (Math.Abs(p-1400000)<50000)
						{
							dcrit=0.92/1000.0;
							k=0.625;
						}
					}
					if (Math.Abs(d-0.00288)<0.00001)	
					{
						if (Math.Abs(p-600000)<50000)	
						{
							dcrit=0.87/1000.0;
							k=0.181;
						}
						if (Math.Abs(p-1000000)<50000)
						{
							dcrit=0.98/1000.0;
							k=0.371;
						}
						if (Math.Abs(p-1400000)<50000)
						{
							dcrit=1.03/1000.0;
							k=0.624;
						}
					}
					if (Math.Abs(d-0.00426)<0.00001)
					{
						if (Math.Abs(p-600000)<50000)	
						{
							dcrit=1.05/1000.0;
							k=0.299;
						}
						if (Math.Abs(p-1000000)<50000)	
						{
							dcrit=1.10/1000.0;
							k=0.391;
						}
						if (Math.Abs(p-1400000)<50000)	
						{
							dcrit=1.18/1000.0;
							k=0.562;
						}
					}
					break;
				}
				case "Air-Water":
				{
					dcrit=Math.Min(d,dmin);
					break;
				}
			}
			StartPoint=0;
			int i=StartPoint;
			Uh=1.0;
			do
			{
				X[i]=Math.Pow(10,(Math.Log10(MinUgs)+i*Xstep));
				do
				{
					Um=Uh;
					a=Rog-Rol;
					b=Rol*Um+(Rol-Rog)*X[i];
					c=-k*Rol*Um*X[i];
					Ugs[i]=(-b+Math.Sqrt(b*b-4.0*a*c))/(2.0*a);
					actualVoidFraction=Ugs[i]/Um;
					f=this.getFrictionCoefficient(Um);
					We=Rog*dcrit*Um*Um/St;
					Uh=A*(1.0+B*Math.Pow(actualVoidFraction*Math.Pow(We,m),n))*Math.Sqrt(St/(f*Rol*dcrit));
				}while (Math.Abs(Um/Uh-1)>0.001);
				Uls[i]=Uh-Ugs[i];
				Y[i]=k*X[i]*Uh/Ugs[i]-X[i];
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				DispersedbubbleBubblyBoundaryF[i].X=(float)X[i];
				DispersedbubbleBubblyBoundaryF[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls && Y[i-1]>MinUls);
			EndPoint=i-2;
			
			int l=0, j=0;
			if (FrmFlowPatternGraph.isBoundaryF3Valid)
			{
				for (l=0; l<1000; l++)
				{
					while (DispersedbubbleBubblyBoundaryF[j].X<=DispersedbubbleChurnBoundaryF[l].X)
					{
						j++;
					}
					if (DispersedbubbleBubblyBoundaryF[j].Y<=DispersedbubbleChurnBoundaryF[l].Y)
					{
						break;
					}
				}
				EndPoint=j;
				if (EndPoint>999)
				{
					EndPoint=998;
				}
			}
		}

		public void GetDispersedbubbleChurnBoundaryF(out int StartPoint,out int EndPoint)
			// Chen. (2006)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)

			StartPoint=0;
			int i=StartPoint;
			int k, m;
			int j=0;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=criticalVoidFraction1*Y[i]/(1-criticalVoidFraction1);
				DispersedbubbleChurnBoundaryF[i].X=(float)X[i];
				DispersedbubbleChurnBoundaryF[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-1;
			if (FrmFlowPatternGraph.isBoundaryF2Valid)
			{
				for (k=0; k<1000; k++)
				{
					while (DispersedbubbleBubblyBoundaryF[j].X<=DispersedbubbleChurnBoundaryF[k].X)
					{
						j++;
					}
					if (DispersedbubbleBubblyBoundaryF[j].Y<=DispersedbubbleChurnBoundaryF[k].Y)
					{
						break;
					}
				}
				StartPoint=k;
				dispersedbubbleChurnBoundaryStartPoint=k;
			}
			if (FrmFlowPatternGraph.isBoundaryF6Valid)
			{
				for (m=0; m<1000; m++)
				{
					if (DispersedbubbleChurnBoundaryF[m].X>ChurnAnnularBoundaryF[0].X)
					{
						break;
					}
				}
				EndPoint=m;
				if (EndPoint>999)
				{
					EndPoint=998;
				}
			}
		}

		public void GetSlugChurnBoundaryF1(out int StartPoint,out int EndPoint)
			//High Uls region, Chen et al. (2006)
		{
			double[] X=new double[GridNumber];	//Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber];	//Liquid superficial velocity (m/s)
			double Um,Uh,f;						//homogeneous velocity (m/s)
			StartPoint=0;
			int i=StartPoint;
			Uh=1.0;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				do
				{
					Um=Uh;
					f=this.getFrictionCoefficient(Um);
					Uh=2.75*Math.Sqrt(St/(f*Rol*d));
				}while (Math.Abs(Um/Uh-1)>0.001);
				X[i]=Uh-Y[i];
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				SlugChurnBoundaryF1[i].X=(float)X[i];
				SlugChurnBoundaryF1[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls && X[i-1]>MinUgs);
			EndPoint=i-2;

			int l1, j1=0;
			if (FrmFlowPatternGraph.isBoundaryF5Valid)
			{
				for (l1=0; l1<1000; l1++)
				{
					while (SlugChurnBoundaryF1[j1].Y<=SlugChurnBoundaryF2[l1].Y)
					{
						j1++;
					}
					if (SlugChurnBoundaryF1[j1].X<=SlugChurnBoundaryF2[l1].X)
					{
						break;
					}
				}
				StartPoint=j1;
			}

			int l2, j2=EndPoint;
			if (FrmFlowPatternGraph.isBoundaryF1Valid)
			{
				j2=0;
				for (l2=0; l2<1000; l2++)
				{
					while (SlugChurnBoundaryF1[j2].Y<=BubblySlugBoundaryF[l2].Y)
					{
						j2++;
					}
					if (SlugChurnBoundaryF1[j2].X<=BubblySlugBoundaryF[l2].X)
					{
						break;
					}
				}
			}

			int l3, j3=EndPoint;
			if (FrmFlowPatternGraph.isBoundaryF2Valid)
			{
				for (l3=0; l3<1000; l3++)
				{
					while (SlugChurnBoundaryF1[j3].X<=DispersedbubbleBubblyBoundaryF[l3].X)
					{
						j3--;
					}
					if (SlugChurnBoundaryF1[j3].Y<=DispersedbubbleBubblyBoundaryF[l3].Y)
					{
						break;
					}
				}
			}
			EndPoint=Math.Min(j2,j3);
		}

		public void GetSlugChurnBoundaryF2(out int StartPoint,out int EndPoint)
			//Middle Uls region, Chen et al. (2006)
		{
			double[] X=new double[GridNumber];	//Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber];	//Liquid superficial velocity (m/s)
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=Math.Pow(Mul*Math.Pow(St,-0.6523)*Math.Pow(10,1.927),1.0/(1-0.6523));
				X[i]=Math.Sqrt(X[i]/(Rog*d));
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				SlugChurnBoundaryF2[i].X=(float)X[i];
				SlugChurnBoundaryF2[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs && Y[i-1]<MaxUls);
			EndPoint=i-2;

			int l, j=0;
			if (FrmFlowPatternGraph.isBoundaryF4Valid)
			{
				for (l=0; l<1000; l++)
				{
					while (SlugChurnBoundaryF2[j].Y<=SlugChurnBoundaryF1[l].Y)
					{
						j++;
					}
					if (SlugChurnBoundaryF2[j].X>=SlugChurnBoundaryF1[l].X)
					{
						break;
					}
				}
				EndPoint=j;
			}
		}

		public void GetChurnAnnularBoundaryF(out int StartPoint,out int EndPoint)
			//High Ugs and Uls region, Chen et al. (2006)
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=Math.Sqrt(Math.Pow(10,5.494)*Math.Sqrt(g/d)*Mug/Rog);
				ChurnAnnularBoundaryF[i].X=(float)X[i];
				ChurnAnnularBoundaryF[i].Y=(float)Y[i];
				i++;
			}while (Y[i-1]<MaxUls);
			EndPoint=i-1;
		}

		public void GetConstantQualityBoundary(out int StartPoint,out int EndPoint, double Quality)
			//quality is constant value
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)

			StartPoint=0;
			int i=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=Y[i]*(Quality*Rol)/((1-Quality)*Rog);
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				ConstantQualityBoundary[i].X=(float)X[i];
				ConstantQualityBoundary[i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs/10.0 && Y[i-1]<MaxUls);
			EndPoint=i-1;
		}

		public void GetConstantQualitiesBoundary(out int StartPoint,out int EndPoint,double Quality,int n)
			//a serious of constant quality lines
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			ConstantQualitiesBoundary[n]=new PointF[GridNumber];
			StartPoint=0;
			int i=StartPoint;
			do
			{
				Y[i]=Math.Pow(10,(Math.Log10(MinUls)+i*Ystep));
				X[i]=Y[i]*(Quality*Rol)/((1-Quality)*Rog);
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]>=MinUls) StartPoint=i;
				ConstantQualitiesBoundary[n][i].X=(float)X[i];
				ConstantQualitiesBoundary[n][i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs/10.0 && Y[i-1]<MaxUls*0.8);
			EndPoint=i-1;
		}

		public void GetConstantMassFluxBoundary(out int StartPoint,out int EndPoint,double MassFlux,int n)
			//a serious of constant mass flux lines
		{
			double[] X=new double[GridNumber]; //Vapour superficial velocity (m/s)
			double[] Y=new double[GridNumber]; //Liquid superficial velocity (m/s)
			ConstantMassFluxBoundary[n]=new PointF[GridNumber];
			StartPoint=0;
			int i=StartPoint;
			do
			{
				X[i]=Math.Pow(10,(Math.Log10(MinUgs)+i*Xstep));
				Y[i]=(MassFlux-X[i]*Rog)/Rol;
				if (StartPoint==0 && X[i]>=MinUgs && Y[i]<=MaxUls) StartPoint=i;
				ConstantMassFluxBoundary[n][i].X=(float)X[i];
				ConstantMassFluxBoundary[n][i].Y=(float)Y[i];
				i++;
			}while (X[i-1]<MaxUgs/10.0 && Y[i-1]<MaxUls*0.6 && Y[i-1]>MinUls);
			EndPoint=i-2;
		}
	}
}
